var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
* Copyright 2018 ARDUINO SA (http://www.arduino.cc/)
* This file is part of arduino-create-agent-js-client.
* Copyright (c) 2018
* Authors: Alberto Iannaccone, Stefania Mellai, Gabriele Destefanis
*
* This software is released under:
* The GNU General Public License, which covers the main part of
* arduino-create-agent-js-client
* The terms of this license can be found at:
* https://www.gnu.org/licenses/gpl-3.0.en.html
*
* You can be released from the requirements of the above licenses by purchasing
* a commercial license. Buying such a license is mandatory if you want to modify or
* otherwise use the software for commercial activities involving the Arduino
* software without disclosing the source code of your own applications. To purchase
* a commercial license, send an email to license@arduino.cc.
*
*/

import io from 'socket.io-client';
import semVerCompare from 'semver-compare';
import { detect } from 'detect-browser';

import { timer } from 'rxjs';
import { filter, takeUntil, first } from 'rxjs/operators';

import Daemon from './daemon';

// Required agent version
var MIN_VERSION = '1.1.76';
var browser = detect();
var POLLING_INTERVAL = 2500;
var UPLOAD_DONE_TIMER = 5000;

var PROTOCOL = {
  HTTP: 'http',
  HTTPS: 'https'
};

var LOOPBACK_ADDRESS = PROTOCOL.HTTP + '://127.0.0.1';
var LOOPBACK_HOST = PROTOCOL.HTTPS + '://localhost';
var LOOKUP_PORT_START = 8991;
var LOOKUP_PORT_END = 9000;
var orderedPluginAddresses = [LOOPBACK_ADDRESS, LOOPBACK_HOST];

var CANT_FIND_AGENT_MESSAGE = 'Arduino Create Agent cannot be found';

var updateAttempts = 0;

if (browser.name !== 'chrome' && browser.name !== 'firefox') {
  orderedPluginAddresses = [LOOPBACK_HOST, LOOPBACK_ADDRESS];
}

var SocketDaemon = function (_Daemon) {
  _inherits(SocketDaemon, _Daemon);

  function SocketDaemon() {
    _classCallCheck(this, SocketDaemon);

    var _this = _possibleConstructorReturn(this, (SocketDaemon.__proto__ || Object.getPrototypeOf(SocketDaemon)).call(this));

    _this.selectedProtocol = PROTOCOL.HTTP;
    _this.socket = null;
    _this.pluginURL = null;

    _this.openChannel(function () {
      return _this.socket.emit('command', 'list');
    });

    _this.agentFound.subscribe(function (agentFound) {
      if (agentFound) {
        _this._wsConnect();
      } else {
        _this.findAgent();
      }
    });
    return _this;
  }

  _createClass(SocketDaemon, [{
    key: 'initSocket',
    value: function initSocket() {
      var _this2 = this;

      this.socket.on('message', function (message) {
        try {
          _this2.appMessages.next(JSON.parse(message));
        } catch (SyntaxError) {
          _this2.appMessages.next(message);
        }
      });
    }
  }, {
    key: 'notifyDownloadError',
    value: function notifyDownloadError(err) {
      this.downloading.next({ status: this.DOWNLOAD_ERROR, err: err });
    }

    /**
     * Look for the agent endpoint.
     * First search in LOOPBACK_ADDRESS, after in LOOPBACK_HOST if in Chrome or Firefox, otherwise vice versa.
     */

  }, {
    key: 'findAgent',
    value: function findAgent() {
      var _this3 = this;

      this._tryPorts(orderedPluginAddresses[0]).catch(function () {
        return _this3._tryPorts(orderedPluginAddresses[1]);
      }).then(function () {
        return _this3.agentFound.next(true);
      }).catch(function () {
        return timer(POLLING_INTERVAL).subscribe(function () {
          return _this3.findAgent();
        });
      });
    }

    /**
     * Try ports for the selected host. From LOOKUP_PORT_START to LOOKUP_PORT_END
     * @param {string} host - The host value (LOOPBACK_ADDRESS or LOOPBACK_HOST).
     * @return {Promise} info - A promise resolving with the agent info values.
     */

  }, {
    key: '_tryPorts',
    value: function _tryPorts(host) {
      var _this4 = this;

      var pluginLookups = [];

      for (var port = LOOKUP_PORT_START; port < LOOKUP_PORT_END; port += 1) {
        pluginLookups.push(fetch(host + ':' + port + '/info').then(function (response) {
          return response.json().then(function (data) {
            return { response: response, data: data };
          });
        }).catch(function () {
          return Promise.resolve(false);
        }));
        // We expect most of those call to fail, because there's only one agent
        // So we have to resolve them with a false value to let the Promise.all catch all the deferred data
      }

      return Promise.all(pluginLookups).then(function (responses) {
        var found = responses.some(function (r) {
          if (r && r.response && r.response.status === 200) {
            _this4.agentInfo = r.data;

            if (_this4.agentInfo.update_url.indexOf('downloads.arduino.cc') === -1) {
              _this4.error.next('unofficial plugin');
            }

            if (r.response.url.indexOf(PROTOCOL.HTTPS) === 0) {
              _this4.selectedProtocol = PROTOCOL.HTTPS;
            } else {
              // Protocol http, force 127.0.0.1 for old agent versions too
              _this4.agentInfo[_this4.selectedProtocol] = _this4.agentInfo[_this4.selectedProtocol].replace('localhost', '127.0.0.1');
            }
            _this4.pluginURL = _this4.agentInfo[_this4.selectedProtocol];
            return true;
          }
          return false;
        });

        if (found) {
          if (_this4.agentInfo.version && (semVerCompare(_this4.agentInfo.version, MIN_VERSION) >= 0 || _this4.agentInfo.version.indexOf('dev') !== -1)) {
            return _this4.agentInfo;
          }

          updateAttempts += 1;
          if (updateAttempts === 0) {
            return _this4.update();
          }
          if (updateAttempts < 3) {
            return timer(10000).subscribe(function () {
              return _this4.update();
            });
          }
          _this4.error.next('plugin version incompatible');
          return Promise.reject(new Error('plugin version incompatible'));
        }

        // Set channelOpen false for the first time
        if (_this4.channelOpen.getValue() === null) {
          _this4.channelOpen.next(false);
        }
        return Promise.reject(new Error(CANT_FIND_AGENT_MESSAGE + ' at ' + host));
      });
    }

    /**
     * Uses the websocket protocol to connect to the agent
     */

  }, {
    key: '_wsConnect',
    value: function _wsConnect() {
      var _this5 = this;

      var wsProtocol = this.selectedProtocol === PROTOCOL.HTTPS ? 'wss' : 'ws';
      var address = this.agentInfo[wsProtocol];

      // Reset
      if (this.socket) {
        this.socket.destroy();
        delete this.socket;
        this.socket = null;
      }

      this.socket = io(address);

      this.socket.on('connect', function () {
        // On connect download windows drivers which are indispensable for detection of boards
        _this5.downloadTool('windows-drivers', 'latest', 'arduino');
        _this5.downloadTool('bossac', '1.7.0', 'arduino');

        _this5.initSocket();

        _this5.channelOpen.next(true);
      });

      this.socket.on('error', function (error) {
        return _this5.error.next(error);
      });

      this.socket.on('disconnect', function () {
        _this5.channelOpen.next(false);
      });
    }
  }, {
    key: 'handleAppMessage',
    value: function handleAppMessage(message) {
      // Result of a list command
      if (message.Ports) {
        this.handleListMessage(message);
      }
      // Serial monitor message
      if (message.D) {
        this.serialMonitorMessages.next(message.D);
      }

      if (message.ProgrammerStatus) {
        this.handleUploadMessage(message);
      }

      if (message.DownloadStatus) {
        this.handleDownloadMessage(message);
      }

      if (message.Err) {
        this.uploading.next({ status: this.UPLOAD_ERROR, err: message.Err });
      }

      if (message.Error) {
        if (message.Error.indexOf('trying to close') !== -1) {
          // https://github.com/arduino/arduino-create-agent#openclose-ports
          this.serialMonitorOpened.next(false);
        }
      }
    }
  }, {
    key: 'handleListMessage',
    value: function handleListMessage(message) {
      var lastDevices = this.devicesList.getValue();
      if (message.Network && !Daemon.devicesListAreEquals(lastDevices.network, message.Ports)) {
        this.devicesList.next({
          serial: lastDevices.serial,
          network: message.Ports
        });
      } else if (!message.Network && !Daemon.devicesListAreEquals(lastDevices.serial, message.Ports)) {
        this.devicesList.next({
          serial: message.Ports,
          network: lastDevices.network
        });
      }
    }

    /**
     * Check the agent version and call the update if needed.
     */

  }, {
    key: 'update',
    value: function update() {
      var _this6 = this;

      return fetch(this.agentInfo[this.selectedProtocol] + '/update', {
        method: 'POST',
        headers: {
          'Content-Type': 'text/plain; charset=utf-8'
        }
      }).then(function (result) {
        return result.json();
      }).then(function (response) {
        if (!response.ok) {
          if (response && response.error && (response.error.indexOf('proxy') !== -1 || response.error.indexOf('dial tcp') !== -1)) {
            _this6.error.next('proxy error');
            return new Error('proxy error');
          }
        }
        // We reject the promise because the daemon will be restarted, we need to continue looking for the port
        return Promise.reject();
      }).catch(function () {
        console.log('update plugin failed');
      });
    }

    /**
     * Pauses the plugin
     * @return {Promise}
     */

  }, {
    key: 'stopPlugin',
    value: function stopPlugin() {
      if (this.agentFound.getValue()) {
        return fetch(this.agentInfo[this.selectedProtocol] + '/pause', { method: 'POST' });
      }
    }

    /**
     * Send 'close' command to all the available serial ports
     */

  }, {
    key: 'closeAllPorts',
    value: function closeAllPorts() {
      var _this7 = this;

      var devices = this.devicesList.getValue().serial;
      devices.forEach(function (device) {
        _this7.socket.emit('command', 'close ' + device.Name);
      });
    }

    /**
     * Send 'message' to serial port
     * @param {string} port the port name
     * @param {string} message the text to be sent to serial
     */

  }, {
    key: 'writeSerial',
    value: function writeSerial(port, message) {
      this.socket.emit('command', 'send ' + port + ' ' + message);
    }

    /**
     * Request serial port open
     * @param {string} port the port name
     */

  }, {
    key: 'openSerialMonitor',
    value: function openSerialMonitor(port, baudrate) {
      var _this8 = this;

      var serialPort = this.devicesList.getValue().serial.find(function (p) {
        return p.Name === port;
      });
      if (!serialPort) {
        return this.serialMonitorOpened.error(new Error('Can\'t find board at ' + port));
      }
      if (this.uploading.getValue().status === this.UPLOAD_IN_PROGRESS || serialPort.IsOpen) {
        return;
      }

      this.appMessages.pipe(takeUntil(this.serialMonitorOpened.pipe(filter(function (open) {
        return open;
      })))).subscribe(function (message) {
        if (message.Cmd === 'Open') {
          _this8.serialMonitorOpened.next(true);
        }
        if (message.Cmd === 'OpenFail') {
          _this8.serialMonitorOpened.error(new Error('Failed to open serial monitor at ' + port));
        }
      });

      this.socket.emit('command', 'open ' + port + ' ' + baudrate + ' timed');
    }

    /**
     * Request serial port close
     * @param {string} port the port name
     */

  }, {
    key: 'closeSerialMonitor',
    value: function closeSerialMonitor(port) {
      var _this9 = this;

      var serialPort = this.devicesList.getValue().serial.find(function (p) {
        return p.Name === port;
      });
      if (!serialPort) {
        return this.serialMonitorOpened.error(new Error('Can\'t find board at ' + port));
      }
      if (!serialPort.IsOpen) {
        return;
      }

      this.appMessages.pipe(takeUntil(this.serialMonitorOpened.pipe(filter(function (open) {
        return !open;
      })))).subscribe(function (message) {
        if (message.Cmd === 'Close') {
          _this9.serialMonitorOpened.next(false);
        }
        if (message.Cmd === 'CloseFail') {
          _this9.serialMonitorOpened.error(new Error('Failed to close serial monitor at ' + port));
        }
      });
      this.socket.emit('command', 'close ' + port);
    }
  }, {
    key: 'handleUploadMessage',
    value: function handleUploadMessage(message) {
      var _this10 = this;

      if (message.Flash === 'Ok' && message.ProgrammerStatus === 'Done') {
        // After the upload is completed the port goes down for a while, so we have to wait a few seconds
        return timer(UPLOAD_DONE_TIMER).subscribe(function () {
          return _this10.uploading.next({ status: _this10.UPLOAD_DONE, msg: message.Flash });
        });
      }
      switch (message.ProgrammerStatus) {
        case 'Starting':
          this.uploading.next({ status: this.UPLOAD_IN_PROGRESS, msg: 'Programming with: ' + message.Cmd });
          break;
        case 'Busy':
          this.uploading.next({ status: this.UPLOAD_IN_PROGRESS, msg: message.Msg });
          break;
        case 'Error':
          this.uploading.next({ status: this.UPLOAD_ERROR, err: message.Msg });
          break;
        case 'Killed':
          this.uploading.next({ status: this.UPLOAD_IN_PROGRESS, msg: 'terminated by user' });
          this.uploading.next({ status: this.UPLOAD_ERROR, err: 'terminated by user' });
          break;
        case 'Error 404 Not Found':
          this.uploading.next({ status: this.UPLOAD_ERROR, err: message.Msg });
          break;
        default:
          this.uploading.next({ status: this.UPLOAD_IN_PROGRESS, msg: message.Msg });
      }
    }
  }, {
    key: 'handleDownloadMessage',
    value: function handleDownloadMessage(message) {
      switch (message.DownloadStatus) {
        case 'Pending':
          this.downloading.next({ status: this.DOWNLOAD_IN_PROGRESS, msg: message.Msg });
          break;
        case 'Success':
          this.downloading.next({ status: this.DOWNLOAD_DONE, msg: message.Msg });
          break;
        case 'Error':
          this.downloading.next({ status: this.DOWNLOAD_ERROR, err: message.Msg });
          break;
        default:
          this.downloading.next({ status: this.DOWNLOAD_IN_PROGRESS, msg: message.Msg });
      }
    }

    /**
     * Perform an upload via http on the daemon
     * @param {Object} target = {
     *   board: "name of the board",
     *   port: "port of the board",
     *   auth_user: "Optional user to use as authentication",
     *   auth_pass: "Optional pass to use as authentication"
     *   auth_key: "Optional private key",
     *   auth_port: "Optional alternative port (default 22)"
     *   network: true or false
     * }
     * @param {Object} data = {
     *  commandline: "commandline to execute",
        signature: "signature of the commandline",
     *  files: [
     *   {name: "Name of a file to upload on the device", data: 'base64data'}
     *  ],
     *  options: {}
     * }
     */

  }, {
    key: 'upload',
    value: function upload(target, data) {
      var _this11 = this;

      if (!target.network) {
        this.closeSerialMonitor(target.port);
      }
      this.uploading.next({ status: this.UPLOAD_IN_PROGRESS });

      if (data.files.length === 0) {
        // At least one file to upload
        this.uploading.next({ status: this.UPLOAD_ERROR, err: 'You need at least one file to upload' });
        return;
      }

      // Main file
      var file = data.files[0];
      file.name = file.name.split('/');
      file.name = file.name[file.name.length - 1];

      var payload = {
        board: target.board,
        port: target.port,
        commandline: data.commandline,
        signature: data.signature,
        hex: file.data,
        filename: file.name,
        extra: {
          auth: {
            username: target.auth_user,
            password: target.auth_pass,
            private_key: target.auth_key,
            port: target.auth_port
          },
          wait_for_upload_port: data.options.wait_for_upload_port === 'true' || data.options.wait_for_upload_port === true,
          use_1200bps_touch: data.options.use_1200bps_touch === 'true' || data.options.use_1200bps_touch === true,
          network: target.network,
          ssh: target.ssh,
          params_verbose: data.options.param_verbose,
          params_quiet: data.options.param_quiet,
          verbose: data.options.verbose
        },
        extrafiles: data.extrafiles || []
      };

      for (var i = 1; i < data.files.length; i += 1) {
        payload.extrafiles.push({ filename: data.files[i].name, hex: data.files[i].data });
      }

      this.serialMonitorOpened.pipe(filter(function (open) {
        return !open;
      })).pipe(first()).subscribe(function () {
        fetch(_this11.pluginURL + '/upload', {
          method: 'POST',
          headers: {
            'Content-Type': 'text/plain; charset=utf-8'
          },
          body: JSON.stringify(payload)
        }).catch(function (error) {
          _this11.uploading.next({ status: _this11.UPLOAD_ERROR, err: error });
        });
      });
    }

    /**
     * Download tool
     * @param {string} toolName
     * @param {string} toolVersion
     * @param {string} packageName
     * @param {string} replacementStrategy
     */

  }, {
    key: 'downloadTool',
    value: function downloadTool(toolName, toolVersion, packageName) {
      var replacementStrategy = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'keep';

      this.downloading.next({ status: this.DOWNLOAD_IN_PROGRESS });
      this.socket.emit('command', 'downloadtool ' + toolName + ' ' + toolVersion + ' ' + packageName + ' ' + replacementStrategy);
    }

    /**
     * Interrupt upload
     */

  }, {
    key: 'stopUploadCommand',
    value: function stopUploadCommand() {
      this.uploading.next({
        status: this.UPLOAD_ERROR,
        err: 'upload stopped'
      });
      this.socket.emit('command', 'killupload');
    }
  }]);

  return SocketDaemon;
}(Daemon);

export default SocketDaemon;