export var provisioningSketch = {
  verbose: true,
  fqbn: 'arduino:samd:mkr1000',
  sketch: {
    name: 'Provisioning',
    ino: {
      name: 'Provisioning.ino',
      content: '#include <ArduinoIoTCloud.h>\n#include <utility/ECCX08Cert.h>\n#include <utility/ECCX08TLSConfig.h>\n\n#include <ArduinoIoTCloudBearSSL.h>\n#include <ArduinoECCX08.h>\n\nconst bool DEBUG = true;\nconst int keySlot                                   = 0;\nconst int compressedCertSlot                        = 10;\nconst int serialNumberAndAuthorityKeyIdentifierSlot = 11;\nconst int thingIdSlot                               = 12;\n\nvoid setup() {\n  Serial.begin(9600);\n  while (!Serial);\n\n  if (!ECCX08.begin()) {\n    Serial.println("No ECCX08 present!");\n    while (1);\n  }\n\n  if (!ECCX08.locked()) {\n    String lockConfirm = promptAndReadLine("Your ECCX08 is unlocked, would you like to lock it (y/N): ");\n    lockConfirm.toLowerCase();\n\n    if (lockConfirm != "y") {\n      Serial.println("That\'s all folks");\n      while (1);\n    }\n\n    if (!ECCX08.writeConfiguration(DEFAULT_ECCX08_TLS_CONFIG)) {\n      Serial.println("Writing ECCX08 configuration failed!");\n      while (1);\n    }\n\n    if (!ECCX08.lock()) {\n      Serial.println("Locking ECCX08 configuration failed!");\n      while (1);\n    }\n\n    Serial.println("ECCX08 locked successfully");\n    Serial.println();\n  }\n\n  String csrConfirm = promptAndReadLine("Would you like to generate a new private key and CSR (y/N): ");\n  csrConfirm.toLowerCase();\n\n  if (csrConfirm != "y") {\n    Serial.println("That\'s all folks");\n    while (1);\n  }\n\n  if (!ECCX08Cert.beginCSR(keySlot, true)) {\n    Serial.println("Error starting CSR generation!");\n    while (1);\n  }\n\n  String thingId = promptAndReadLine("Please enter the thing id: ");\n  ECCX08Cert.setSubjectCommonName(thingId);\n\n  String csr = ECCX08Cert.endCSR();\n\n  if (!csr) {\n    Serial.println("Error generating CSR!");\n    while (1);\n  }\n\n  Serial.println("Generated CSR is:");\n  Serial.println();\n  Serial.println(csr);\n\n  String issueYear              = promptAndReadLine("Please enter the issue year of the certificate (2000 - 2031): ");\n  String issueMonth             = promptAndReadLine("Please enter the issue month of the certificate (1 - 12): ");\n  String issueDay               = promptAndReadLine("Please enter the issue day of the certificate (1 - 31): ");\n  String issueHour              = promptAndReadLine("Please enter the issue hour of the certificate (0 - 23): ");\n  String expireYears            = promptAndReadLine("Please enter how many years the certificate is valid for (0 - 31): ");\n  String serialNumber           = promptAndReadLine("Please enter the certificates serial number: ");\n  String authorityKeyIdentifier = promptAndReadLine("Please enter the certificates authority key identifier: ");\n  String signature              = promptAndReadLine("Please enter the certificates signature: ");\n\n  byte thingIdBytes[72];\n  byte serialNumberBytes[16];\n  byte authorityKeyIdentifierBytes[20];\n  byte signatureBytes[64];\n\n  thingId.getBytes(thingIdBytes, sizeof(thingIdBytes));\n  hexStringToBytes(serialNumber, serialNumberBytes, sizeof(serialNumberBytes));\n  hexStringToBytes(authorityKeyIdentifier, authorityKeyIdentifierBytes, sizeof(authorityKeyIdentifierBytes));\n  hexStringToBytes(signature, signatureBytes, sizeof(signatureBytes));\n\n  if (!ECCX08.writeSlot(thingIdSlot, thingIdBytes, sizeof(thingIdBytes))) {\n    Serial.println("Error storing thing id!");\n    while (1);\n  }\n\n  if (!ECCX08Cert.beginStorage(compressedCertSlot, serialNumberAndAuthorityKeyIdentifierSlot)) {\n    Serial.println("Error starting ECCX08 storage!");\n    while (1);\n  }\n\n  ECCX08Cert.setSignature(signatureBytes);\n  ECCX08Cert.setAuthorityKeyIdentifier(authorityKeyIdentifierBytes);\n  ECCX08Cert.setSerialNumber(serialNumberBytes);\n  ECCX08Cert.setIssueYear(issueYear.toInt());\n  ECCX08Cert.setIssueMonth(issueMonth.toInt());\n  ECCX08Cert.setIssueDay(issueDay.toInt());\n  ECCX08Cert.setIssueHour(issueHour.toInt());\n  ECCX08Cert.setExpireYears(expireYears.toInt());\n\n  if (!ECCX08Cert.endStorage()) {\n    Serial.println("Error storing ECCX08 compressed cert!");\n    while (1);\n  }\n\n  if (!ECCX08Cert.beginReconstruction(keySlot, compressedCertSlot, serialNumberAndAuthorityKeyIdentifierSlot)) {\n    Serial.println("Error starting ECCX08 cert reconstruction!");\n    while (1);\n  }\n\n  ECCX08Cert.setIssuerCountryName("US");\n  ECCX08Cert.setIssuerOrganizationName("Arduino LLC US");\n  ECCX08Cert.setIssuerOrganizationalUnitName("IT");\n  ECCX08Cert.setIssuerCommonName("Arduino");\n\n  if (!ECCX08Cert.endReconstruction()) {\n    Serial.println("Error reconstructing ECCX08 compressed cert!");\n    while (1);\n  }\n\n  if (!DEBUG) {\n    return;\n  }\n\n  Serial.println("Compressed cert = ");\n\n  const byte* certData = ECCX08Cert.bytes();\n  int certLength = ECCX08Cert.length();\n\n  for (int i = 0; i < certLength; i++) {\n    byte b = certData[i];\n\n    if (b < 16) {\n      Serial.print(\'0\');\n    }\n    Serial.print(b, HEX);\n  }\n  Serial.println();\n}\n\nvoid loop() {\n}\n\nString promptAndReadLine(const char* prompt) {\n  Serial.print(prompt);\n  String s = readLine();\n  Serial.println(s);\n\n  return s;\n}\n\nString readLine() {\n  String line;\n\n  while (1) {\n    if (Serial.available()) {\n      char c = Serial.read();\n\n      if (c == \'\\r\') {\n        // ignore\n      } else if (c == \'\\n\') {\n        break;\n      }\n\n      line += c;\n    }\n  }\n\n  line.trim();\n\n  return line;\n}\n\nvoid hexStringToBytes(String& in, byte out[], int length) {\n  int inLength = in.length();\n  in.toUpperCase();\n  int outLength = 0;\n\n  for (int i = 0; i < inLength && outLength < length; i += 2) {\n    char highChar = in[i];\n    char lowChar = in[i + 1];\n\n    byte highByte = (highChar <= \'9\') ? (highChar - \'0\') : (highChar + 10 - \'A\');\n    byte lowByte = (lowChar <= \'9\') ? (lowChar - \'0\') : (lowChar + 10 - \'A\');\n\n    out[outLength++] = (highByte << 4) | (lowByte & 0xF);\n  }\n}\n'
    },
    metadata: {}
  }
};